@@ -26,16 +26,18 @@
 #include <proxygen/lib/utils/UtilInl.h>
 #include <quic/api/QuicSocket.h>
 #include <quic/client/QuicClientTransport.h>
+#include <quic/common/udpsocket/FollyQuicAsyncUDPSocket.h>
 #include <quic/congestion_control/CongestionControllerFactory.h>
 #include <quic/fizz/client/handshake/FizzClientQuicHandshakeContext.h>
 #include <quic/logging/FileQLogger.h>
 
 namespace quic::samples {
 
-HQClient::HQClient(const HQToolClientParams& params) : params_(params) {
+HQClient::HQClient(const HQToolClientParams& params)
+    : params_(params), qEvb_(std::make_shared<FollyQuicEventBase>(&evb_)) {
   if (params_.transportSettings.pacingEnabled) {
-    pacingTimer_ = TimerHighRes::newTimer(
-        &evb_, params_.transportSettings.pacingTimerTickInterval);
+    pacingTimer_ = std::make_shared<HighResQuicTimer>(
+        &evb_, params_.transportSettings.pacingTimerResolution);
   }
 }
 
@@ -53,7 +55,7 @@ int HQClient::start() {
   evb_.loopForever();
   if (params_.migrateClient) {
     quicClient_->onNetworkSwitch(
-        std::make_unique<folly::AsyncUDPSocket>(&evb_));
+        std::make_unique<FollyQuicAsyncUDPSocket>(qEvb_));
     sendRequests(true, quicClient_->getNumOpenableBidirectionalStreams());
   }
   evb_.loop();
@@ -122,6 +124,7 @@ HQClient::sendRequest(const proxygen::URL& requestUrl) {
   if (!txn) {
     return nullptr;
   }
+
   if (!params_.outdir.empty()) {
     bool canWrite = false;
     // default output file name
@@ -169,14 +172,27 @@ void HQClient::sendRequests(bool closeSession, uint64_t numOpenableStreams) {
   // callback on the first EOM to try to make one more request. That callback
   // will keep scheduling itself until there are no more requests.
   if (params_.sendRequestsSequentially && !httpPaths_.empty()) {
-    auto sendOneMoreRequest = [&]() {
-      uint64_t numOpenable = quicClient_->getNumOpenableBidirectionalStreams();
-      if (numOpenable > 0) {
-        sendRequests(true, numOpenable);
-      };
+    auto callSendRequestsAfterADelay = [&]() {
+      if (params_.gapBetweenRequests.count() > 0) {
+        evb_.runAfterDelay(
+            [&]() {
+              uint64_t numOpenable =
+                  quicClient_->getNumOpenableBidirectionalStreams();
+              if (numOpenable > 0) {
+                sendRequests(true, numOpenable);
+              };
+            },
+            params_.gapBetweenRequests.count());
+      } else {
+        uint64_t numOpenable =
+            quicClient_->getNumOpenableBidirectionalStreams();
+        if (numOpenable > 0) {
+          sendRequests(true, numOpenable);
+        };
+      }
     };
     CHECK(!curls_.empty());
-    curls_.back()->setEOMFunc(sendOneMoreRequest);
+    curls_.back()->setEOMFunc(callSendRequestsAfterADelay);
   }
 }
 static std::function<void()> selfSchedulingRequestRunner;
@@ -245,9 +261,9 @@ void HQClient::connectError(const quic::QuicError& error) {
 }
 
 void HQClient::initializeQuicClient() {
-  auto sock = std::make_unique<folly::AsyncUDPSocket>(&evb_);
+  auto sock = std::make_unique<FollyQuicAsyncUDPSocket>(qEvb_);
   auto client = std::make_shared<quic::QuicClientTransport>(
-      &evb_,
+      qEvb_,
       std::move(sock),
       quic::FizzClientQuicHandshakeContext::Builder()
           .setFizzClientContext(
