@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  * All rights reserved.
  *
  * This source code is licensed under the BSD-style license found in the
@@ -44,10 +44,6 @@ const char* getErrorCodeString(ErrorCode error) {
       return "INADEQUATE_SECURITY";
     case ErrorCode::HTTP_1_1_REQUIRED:
       return "HTTP_1_1_REQUIRED";
-    case ErrorCode::_SPDY_INVALID_STREAM:
-      return "_SPDY_INVALID_STREAM";
-    case ErrorCode::_HTTP3_PR_INVALID_OFFSET:
-      return "_HTTP3_PR_INVALID_OFFSET";
   }
   LOG(FATAL) << "Unreachable";
   return "";
