@@ -13,28 +13,19 @@
 
 namespace proxygen {
 
-class HeadersRateLimitFilter : public RateLimitFilter {
+class HeadersRateLimiter : public RateLimiter {
  public:
   static const uint32_t kDefaultMaxEventsPerInterval = 50000;
   static const uint32_t kMaxEventsPerIntervalLowerBound = 100;
   static constexpr std::chrono::milliseconds kDefaultTimeoutDuration{100};
 
-  explicit HeadersRateLimitFilter(folly::HHWheelTimer* timer,
-                                  HTTPSessionStats* httpSessionStats)
-      : RateLimitFilter(timer, httpSessionStats) {
+  explicit HeadersRateLimiter(folly::HHWheelTimer* timer,
+                              HTTPSessionStats* httpSessionStats)
+      : RateLimiter(timer, httpSessionStats) {
     maxEventsInInterval_ = kDefaultMaxEventsPerInterval;
     timeoutDuration_ = kDefaultTimeoutDuration;
   }
 
-  void onHeadersComplete(StreamID stream,
-                         std::unique_ptr<HTTPMessage> msg) override {
-    if (!incrementNumEventsInCurrentInterval()) {
-      callback_->onHeadersComplete(stream, std::move(msg));
-    } else {
-      callback_->onGoaway(http2::kMaxStreamID, ErrorCode::NO_ERROR);
-    }
-  }
-
   void recordNumEventsInCurrentInterval(uint32_t numEvents) override {
     if (httpSessionStats_) {
       httpSessionStats_->recordHeadersInInterval(numEvents);
