@@ -25,7 +25,7 @@
 #include <proxygen/lib/http/codec/HTTPSettings.h>
 #include <proxygen/lib/http/session/HQByteEventTracker.h>
 #include <proxygen/lib/http/session/HQStreamBase.h>
-#include <proxygen/lib/http/session/HQUnidirectionalCallbacks.h>
+#include <proxygen/lib/http/session/HQStreamDispatcher.h>
 #include <proxygen/lib/http/session/HTTPSessionBase.h>
 #include <proxygen/lib/http/session/HTTPSessionController.h>
 #include <proxygen/lib/http/session/HTTPTransaction.h>
@@ -34,6 +34,7 @@
 #include <proxygen/lib/utils/ConditionalGate.h>
 #include <quic/api/QuicSocket.h>
 #include <quic/common/BufUtil.h>
+#include <quic/common/events/FollyQuicEventBase.h>
 
 namespace proxygen {
 
@@ -82,7 +83,8 @@ class HQSession
     , public quic::QuicSocket::PingCallback
     , public HTTPSessionBase
     , public folly::EventBase::LoopCallback
-    , public HQUnidirStreamDispatcher::Callback {
+    , public HQUniStreamDispatcher::Callback
+    , public HQBidiStreamDispatcher::Callback {
   // Forward declarations
  public:
   class HQStreamTransportBase;
@@ -255,8 +257,10 @@ class HQSession
   }
 
   folly::EventBase* getEventBase() const override {
-    if (sock_) {
-      return sock_->getEventBase();
+    if (sock_ && sock_->getEventBase()) {
+      return sock_->getEventBase()
+          ->getTypedEventBase<quic::FollyQuicEventBase>()
+          ->getBackingEventBase();
     }
     return nullptr;
   }
@@ -277,11 +281,6 @@ class HQSession
     return CodecProtocol::HTTP_3;
   }
 
-  // for testing only
-  HQUnidirStreamDispatcher* getDispatcher() {
-    return &unidirectionalReadDispatcher_;
-  }
-
   const TimePoint& getTransportStart() const {
     return transportStart_;
   }
@@ -333,6 +332,10 @@ class HQSession
     }
   }
 
+  [[nodiscard]] bool supportsWebTransport() const {
+    return supportsWebTransport_.all();
+  }
+
   void setMaxConcurrentIncomingStreams(uint32_t /*num*/) override {
     // need transport API
   }
@@ -352,6 +355,11 @@ class HQSession
     return 0;
   }
 
+  size_t sendPing(uint64_t data) override {
+    sock_->sendPing(std::chrono::milliseconds(data));
+    return 0;
+  }
+
   /**
    * Sends a knob frame on the session.
    */
@@ -433,7 +441,8 @@ class HQSession
     versionUtilsReady_.then([this, stats] { qpackCodec_.setStats(stats); });
   }
 
-  void setHeaderIndexingStrategy(const HeaderIndexingStrategy* indexingStrat) {
+  void setHeaderIndexingStrategy(
+      const HeaderIndexingStrategy* indexingStrat) override {
     versionUtilsReady_.then([this, indexingStrat] {
       qpackCodec_.setHeaderIndexingStrategy(indexingStrat);
     });
@@ -605,37 +614,42 @@ class HQSession
     return nullptr;
   }
 
-  /*
-   * for HQ we need a read callback for unidirectional streams to read the
-   * stream type from the the wire to decide whether a stream is
-   * a control stream, a header codec/decoder stream or a push stream
-   *
-   * This part is now implemented in HQUnidirStreamDispatcher
-   */
+  // Callback methods that are invoked by the stream dispatchers
+  void dispatchControlStream(quic::StreamId /* id */,
+                             hq::UnidirectionalStreamType /* type */,
+                             size_t /* toConsume */) override;
 
-  // Callback methods that are invoked by the dispatcher
-  void assignPeekCallback(
-      quic::StreamId /* id */,
-      hq::UnidirectionalStreamType /* type */,
-      size_t /* toConsume */,
-      quic::QuicSocket::PeekCallback* const /* cb */) override;
+  void dispatchRequestStream(quic::StreamId /* streamId */) override;
+  void dispatchRequestStreamImpl(quic::StreamId /* streamId */);
 
-  void assignReadCallback(
-      quic::StreamId /* id */,
-      hq::UnidirectionalStreamType /* type */,
-      size_t /* toConsume */,
-      quic::QuicSocket::ReadCallback* const /* cb */) override;
+  std::chrono::milliseconds getDispatchTimeout() const override {
+    return transactionsTimeout_;
+  }
 
   void rejectStream(quic::StreamId /* id */) override;
 
-  folly::Optional<hq::UnidirectionalStreamType> parseStreamPreface(
+  folly::Optional<hq::UnidirectionalStreamType> parseUniStreamPreface(
       uint64_t preface) override;
 
-  void controlStreamReadAvailable(quic::StreamId /* id */) override;
+  folly::Optional<hq::BidirectionalStreamType> parseBidiStreamPreface(
+      uint64_t preface) override {
+    if (preface ==
+        folly::to_underlying(hq::BidirectionalStreamType::WEBTRANSPORT)) {
+      if (supportsWebTransport()) {
+        return hq::BidirectionalStreamType::WEBTRANSPORT;
+      } else {
+        LOG(ERROR) << "WT stream when it is unsupported sess=" << *this;
+        return folly::none;
+      }
+    }
+    if (direction_ == TransportDirection::DOWNSTREAM) {
+      return hq::BidirectionalStreamType::REQUEST;
+    }
+    return folly::none;
+  }
 
-  void controlStreamReadError(
-      quic::StreamId /* id */,
-      const HQUnidirStreamDispatcher::Callback::ReadError& /* err */) override;
+  HQStreamTransportBase* FOLLY_NULLABLE
+  findWTSessionOrAbort(quic::StreamId sessionID, quic::StreamId streamId);
 
   /**
    * HQSession is an HTTPSessionBase that uses QUIC as the underlying transport
@@ -665,6 +679,8 @@ class HQSession
         dropping_(false),
         inLoopCallback_(false),
         unidirectionalReadDispatcher_(*this, direction),
+        bidirectionalReadDispatcher_(*this, direction),
+        controlStreamReadCallback_(*this),
         sessionObserverAccessor_(this),
         sessionObserverContainer_(&sessionObserverAccessor_) {
     codec_.add<HTTPChecks>();
@@ -675,6 +691,7 @@ class HQSession
     attachToSessionController();
     nextEgressResults_.reserve(maxConcurrentIncomingStreams_);
     quicInfo_ = std::make_shared<QuicProtocolInfo>();
+    initCodecHeaderIndexingStrategy();
   }
 
   // EventBase::LoopCallback methods
@@ -749,8 +766,6 @@ class HQSession
   HQStreamTransport* createStreamTransport(quic::StreamId streamId);
 
   bool createEgressControlStreams();
-  HQControlStream* tryCreateIngressControlStream(quic::StreamId id,
-                                                 uint64_t preface);
 
   // Creates outgoing control stream.
   bool createEgressControlStream(hq::UnidirectionalStreamType streamType);
@@ -1045,6 +1060,22 @@ class HQSession
     bool readEOF_{false};
   }; // HQControlStream
 
+  // Callback for the control stream - follows the read api
+  struct ControlStreamReadCallback : public quic::QuicSocket::ReadCallback {
+    explicit ControlStreamReadCallback(HQSession& session) : session_(session) {
+    }
+    ~ControlStreamReadCallback() override = default;
+    void readAvailable(quic::StreamId id) noexcept override {
+      session_.controlStreamReadAvailable(id);
+    }
+    void readError(quic::StreamId id, quic::QuicError error) noexcept override {
+      session_.controlStreamReadError(id, error);
+    }
+
+   protected:
+    HQSession& session_;
+  };
+
  public:
   class HQStreamTransportBase
       : public HQStreamBase
@@ -1128,7 +1159,7 @@ class HQSession
       auto len = chain->computeChainDataLength();
       if (session_.onBodyImpl(std::move(chain), len, padding, &txn_)) {
         session_.pauseReads();
-      };
+      }
     }
 
     void onChunkHeader(HTTPCodec::StreamID /* stream */,
@@ -1368,6 +1399,10 @@ class HQSession
       return session_.transportInfo_;
     }
 
+    [[nodiscard]] std::chrono::seconds getLatestIdleTime() const override {
+      return session_.getLatestIdleTime();
+    }
+
     bool getCurrentTransportInfo(wangle::TransportInfo* tinfo) override;
 
     void getFlowControlInfo(
@@ -1718,6 +1753,14 @@ class HQSession
     folly::Optional<hq::PushId> ingressPushId_;
   }; // HQStreamTransportBase
 
+  void dispatchUniWTStream(quic::StreamId /* streamId */,
+                           quic::StreamId /* sessionId */,
+                           size_t /* to consume */) override;
+
+  void dispatchBidiWTStream(quic::StreamId /* streamId */,
+                            quic::StreamId /* sessionId */,
+                            size_t /* to consume */) override;
+
  protected:
   class HQStreamTransport
       : public detail::singlestream::SSBidir
@@ -1765,6 +1808,80 @@ class HQSession
 
     uint16_t getDatagramSizeLimit() const noexcept override;
     bool sendDatagram(std::unique_ptr<folly::IOBuf> datagram) override;
+
+    [[nodiscard]] bool supportsWebTransport() const override {
+      return session_.supportsWebTransport();
+    }
+    folly::Expected<HTTPCodec::StreamID, WebTransport::ErrorCode>
+    newWebTransportBidiStream() override;
+    folly::Expected<HTTPCodec::StreamID, WebTransport::ErrorCode>
+    newWebTransportUniStream() override;
+
+    folly::Expected<HTTPTransaction::Transport::FCState,
+                    WebTransport::ErrorCode>
+    sendWebTransportStreamData(HTTPCodec::StreamID /*id*/,
+                               std::unique_ptr<folly::IOBuf> /*data*/,
+                               bool /*eof*/) override;
+
+    folly::Expected<folly::Unit, WebTransport::ErrorCode>
+        resetWebTransportEgress(HTTPCodec::StreamID /*id*/,
+                                uint32_t /*errorCode*/) override;
+
+    folly::Expected<folly::Unit, WebTransport::ErrorCode>
+        pauseWebTransportIngress(HTTPCodec::StreamID /*id*/) override;
+
+    folly::Expected<folly::Unit, WebTransport::ErrorCode>
+        resumeWebTransportIngress(HTTPCodec::StreamID /*id*/) override;
+
+    folly::Expected<folly::Unit, WebTransport::ErrorCode>
+        stopReadingWebTransportIngress(HTTPCodec::StreamID /*id*/,
+                                       uint32_t /*errorCode*/) override;
+
+    class WTWriteCallback : public quic::QuicSocket::WriteCallback {
+     public:
+      explicit WTWriteCallback(HTTPTransaction& txn) : txn_(txn) {
+      }
+
+      void onStreamWriteReady(quic::StreamId id, uint64_t) noexcept override {
+        VLOG(4) << "onStreamWriteReady id=" << id;
+        txn_.onWebTransportEgressReady(id);
+      }
+
+     private:
+      HTTPTransaction& txn_;
+    };
+
+    class WTReadCallback : public quic::QuicSocket::ReadCallback {
+     public:
+      explicit WTReadCallback(HTTPTransaction& txn, HQSession& session)
+          : txn_(txn), session_(session) {
+      }
+
+      void readAvailable(quic::StreamId id) noexcept override;
+
+      void readError(quic::StreamId id,
+                     quic::QuicError error) noexcept override;
+
+     private:
+      HTTPTransaction& txn_;
+      HQSession& session_;
+    };
+
+    std::unique_ptr<WTWriteCallback> wtWriteCallback_;
+    std::unique_ptr<WTReadCallback> wtReadCallback_;
+
+    WTWriteCallback* getWTWriteCallback() {
+      if (!wtWriteCallback_) {
+        wtWriteCallback_ = std::make_unique<WTWriteCallback>(txn_);
+      }
+      return wtWriteCallback_.get();
+    }
+    WTReadCallback* getWTReadCallback() {
+      if (!wtReadCallback_) {
+        wtReadCallback_ = std::make_unique<WTReadCallback>(txn_, session_);
+      }
+      return wtReadCallback_.get();
+    }
   }; // HQStreamTransport
 
 #ifdef _MSC_VER
@@ -1772,7 +1889,7 @@ class HQSession
 #endif
 
   std::unique_ptr<HTTPCodec> createCodec(quic::StreamId id);
-  bool checkNewStream(quic::StreamId id);
+  bool maybeRejectRequestAfterGoaway(quic::StreamId id);
 
  private:
   void sendGoaway();
@@ -1813,18 +1930,23 @@ class HQSession
   // Remove all callbacks from a stream during cleanup
   void clearStreamCallbacks(quic::StreamId /* id */);
 
+  void controlStreamReadAvailable(quic::StreamId id);
+  void controlStreamReadError(quic::StreamId id, const quic::QuicError& error);
+
   using ControlStreamsKey = std::pair<quic::StreamId, hq::StreamDirection>;
   std::unordered_map<hq::UnidirectionalStreamType, HQControlStream>
       controlStreams_;
-  HQUnidirStreamDispatcher unidirectionalReadDispatcher_;
+  HQUniStreamDispatcher unidirectionalReadDispatcher_;
+  HQBidiStreamDispatcher bidirectionalReadDispatcher_;
+  ControlStreamReadCallback controlStreamReadCallback_;
   QPACKCodec qpackCodec_;
 
   // Min Stream ID we haven't seen so far
   quic::StreamId minUnseenIncomingStreamId_{0};
-  // Maximum Stream ID that we are allowed to open, according to the remote
-  quic::StreamId minPeerUnseenId_{hq::kMaxClientBidiStreamId};
   // Whether SETTINGS have been received
   bool receivedSettings_{false};
+  enum class SettingEnabled : uint8_t { SELF = 0, PEER = 1 };
+  std::bitset<2> supportsWebTransport_;
 
   /**
    * The maximum number of concurrent transactions that this session's peer
@@ -1856,6 +1978,8 @@ class HQSession
        hq::kDefaultEgressQpackBlockedStream},
   };
   HTTPSettings ingressSettings_;
+  // Maximum Stream/Push ID that we are allowed to open, from GOAWAY
+  quic::StreamId peerMinUnseenId_{hq::kMaxClientBidiStreamId};
   uint64_t minUnseenIncomingPushId_{0};
 
   ReadyGate versionUtilsReady_;
@@ -1900,6 +2024,13 @@ class HQSession
     }
     ~ObserverAccessor() override = default;
 
+    size_t sendPing(uint64_t data) override {
+      if (sessionBasePtr_) {
+        return sessionBasePtr_->sendPing(data);
+      }
+      return 0;
+    }
+
    private:
     HTTPSessionBase* sessionBasePtr_{nullptr};
   };
