@@ -119,6 +119,7 @@ class MockHTTPTransactionTransport : public HTTPTransaction::Transport {
         ->getPeerAddressNonConst();
   }
   MOCK_METHOD(void, describe, (std::ostream&), (const));
+  MOCK_METHOD((std::chrono::seconds), getLatestIdleTime, (), (const));
 
   MOCK_METHOD((const wangle::TransportInfo&),
               getSetupTransportInfoNonConst,
@@ -222,6 +223,32 @@ class MockHTTPTransactionTransport : public HTTPTransaction::Transport {
         ->getDatagramSizeLimitNonConst();
   }
 
+  MOCK_METHOD((bool), supportsWebTransport, (), (const));
+  MOCK_METHOD((folly::Expected<HTTPCodec::StreamID, WebTransport::ErrorCode>),
+              newWebTransportBidiStream,
+              ());
+
+  MOCK_METHOD((folly::Expected<HTTPCodec::StreamID, WebTransport::ErrorCode>),
+              newWebTransportUniStream,
+              ());
+  MOCK_METHOD((folly::Expected<FCState, WebTransport::ErrorCode>),
+              sendWebTransportStreamData,
+              (HTTPCodec::StreamID, std::unique_ptr<folly::IOBuf>, bool));
+  MOCK_METHOD((folly::Expected<folly::Unit, WebTransport::ErrorCode>),
+              resetWebTransportEgress,
+              (HTTPCodec::StreamID, uint32_t));
+
+  MOCK_METHOD((folly::Expected<folly::Unit, WebTransport::ErrorCode>),
+              pauseWebTransportIngress,
+              (HTTPCodec::StreamID));
+
+  MOCK_METHOD((folly::Expected<folly::Unit, WebTransport::ErrorCode>),
+              resumeWebTransportIngress,
+              (HTTPCodec::StreamID));
+  MOCK_METHOD((folly::Expected<folly::Unit, WebTransport::ErrorCode>),
+              stopReadingWebTransportIngress,
+              (HTTPCodec::StreamID, uint32_t));
+
   MOCK_METHOD(void, trackEgressBodyOffset, (uint64_t, ByteEvent::EventFlags));
 
   MockHTTPCodec mockCodec_;
@@ -266,7 +293,7 @@ class MockHTTPTransaction : public HTTPTransaction {
     EXPECT_CALL(mockTransport_, getPeerAddressNonConst())
         .WillRepeatedly(testing::ReturnRef(defaultAddress_));
     EXPECT_CALL(mockTransport_, getCodecNonConst())
-        .WillRepeatedly(testing::ReturnRef(mockCodec_));
+        .WillRepeatedly(testing::ReturnRef(mockTransport_.mockCodec_));
     EXPECT_CALL(mockTransport_, getSetupTransportInfoNonConst())
         .WillRepeatedly(testing::ReturnRef(setupTransportInfo_));
     EXPECT_CALL(mockTransport_, getUnderlyingTransportNonConst())
@@ -383,19 +410,18 @@ class MockHTTPTransaction : public HTTPTransaction {
   MOCK_METHOD(void, updateAndSendPriority, (HTTPPriority));
   MOCK_METHOD((bool), addBufferMeta, (), (noexcept));
   void enablePush() {
-    EXPECT_CALL(mockCodec_, supportsPushTransactions())
+    EXPECT_CALL(mockTransport_.mockCodec_, supportsPushTransactions())
         .WillRepeatedly(testing::Return(true));
   }
 
   void setupCodec(CodecProtocol protocol) {
-    EXPECT_CALL(mockCodec_, getProtocol())
+    EXPECT_CALL(mockTransport_.mockCodec_, getProtocol())
         .WillRepeatedly(testing::Return(protocol));
   }
   testing::NiceMock<MockHTTPTransactionTransport> mockTransport_;
   testing::NiceMock<folly::test::MockAsyncTransport> mockAsyncTransport_;
   testing::NiceMock<MockAsyncTransportCertificate> mockPeerCertificate_;
   const folly::SocketAddress defaultAddress_;
-  MockHTTPCodec mockCodec_;
   wangle::TransportInfo setupTransportInfo_;
 };
 
