@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  * All rights reserved.
  *
  * This source code is licensed under the BSD-style license found in the
@@ -8,7 +8,7 @@
 
 #pragma once
 
-#include "proxygen/lib/stats/BaseStats.h"
+#include <proxygen/lib/stats/BaseStats.h>
 
 namespace proxygen {
 
