@@ -398,6 +398,7 @@ TEST(HTTPMessage, TestHeaderStripPerHop) {
   msg.getHeaders().add(HTTP_HEADER_CONNECTION, ", e");
   msg.getHeaders().add(HTTP_HEADER_CONNECTION, " f ,\tg\t, \r\n\th ");
   msg.getHeaders().add("Keep-Alive", "true");
+  msg.getHeaders().add("Priority", "u=5, i");
 
   msg.getHeaders().add("a", "1");
   msg.getHeaders().add("b", "2");
@@ -408,10 +409,13 @@ TEST(HTTPMessage, TestHeaderStripPerHop) {
   msg.getHeaders().add("g", "7");
   msg.getHeaders().add("h", "8");
 
-  EXPECT_EQ(msg.getHeaders().size(), 15);
+  EXPECT_EQ(msg.getHeaders().size(), 16);
   msg.stripPerHopHeaders();
-  EXPECT_EQ(msg.getHeaders().size(), 0);
+  EXPECT_EQ(msg.getHeaders().size(), 1);
   EXPECT_EQ(msg.getStrippedPerHopHeaders().size(), 15);
+  msg.stripPerHopHeaders(/* stripPriority */ true);
+  EXPECT_EQ(msg.getHeaders().size(), 0);
+  EXPECT_EQ(msg.getStrippedPerHopHeaders().size(), 1);
 }
 
 TEST(HTTPMessage, TestEmptyName) {
@@ -761,6 +765,17 @@ TEST(HTTPMessage, HTTPPrioritySetOutRangeUrgency) {
   EXPECT_TRUE(priHeaderViaGetter->incremental);
 }
 
+TEST(HTTPMessage, HTTPPriorityOrderIdSetGet) {
+  HTTPMessage message;
+  message.setHTTPPriority(HTTPPriority(7, false, 10));
+  EXPECT_EQ(httpPriorityFromHTTPMessage(message)->orderId, 10);
+  auto& priHeader = message.getHeaders().getSingleOrEmpty(HTTP_HEADER_PRIORITY);
+  EXPECT_EQ("u=7,o=10", priHeader);
+  auto priHeaderViaGetter = message.getHTTPPriority();
+  EXPECT_EQ(priHeaderViaGetter->urgency, 7);
+  EXPECT_EQ(priHeaderViaGetter->orderId, 10);
+}
+
 TEST(HTTPHeaders, GetSetOnResize) {
   HTTPHeaders headers;
   for (size_t i = 0; i < kInitialVectorReserve - 1; i++) {
