@@ -73,6 +73,24 @@ class BaseStats {
   // At a minimum they require formatters and map lookups for
   // operations and make it easier to violate the constraint that all counters
   // are created at startup.
+
+  // Following are helpers to add/increment optional BaseStats types
+  template <typename StatT>
+  static void addToOptionalStat(StatT& tlStat, int64_t value) {
+    if (tlStat) {
+      tlStat->add(value);
+    }
+  }
+
+  template <typename StatT>
+  static void incrementOptionalCounter(std::optional<StatT>& tlCounter,
+                                       facebook::fb303::CounterType value) {
+    if (tlCounter) {
+      tlCounter->incrementValue(value);
+    }
+  }
+
+  static bool isAllTimeTimeseriesEnabled();
 };
 
 } // namespace proxygen
