@@ -12,6 +12,7 @@
 #include <folly/portability/GMock.h>
 #include <folly/portability/GTest.h>
 
+#include <proxygen/facebook/lib/statistics/ARLResourceStats.h>
 #include <proxygen/lib/stats/ResourceData.h>
 
 using namespace proxygen;
@@ -52,6 +53,8 @@ class MockResources : public Resources {
 
     data.setCpuStats(
         params.cpuUtilRatio,
+        std::vector<double>(params.numCpuCores, params.cpuUtilRatio),
+        80,
         params.cpuSoftIrqUtilRatio,
         std::vector<double>(params.numCpuCores, params.cpuSoftIrqUtilRatio));
     uint64_t totalMemBytes = 100;
